/*
   Here is an unambiguous, left associative, BNF grammar
   (and an EBNF grammar derived from it) for the language

    L3 = {a, (a), a+a, (a)+a, a+(a), (a+a), a+a+a, (a+a)+a, a+(a+a), ((a+a)+a)+a, ...}

    BNF:   expr -> expr '+' term | term  // notice that this is left recursive
           term -> a | '(' expr ')'

    EBNF:  expr -> term ( '+' term )*    // we eliminated the recursion
           term -> a | '(' expr ')'


   We cannot write a recusive descent parser using the BNF grammar
   because the left recursion would cause infinite recursion
   (the getExpr() method would immediately, and repeatedly, call
   itself).

   But we can use the EBNF grammar to write a recursive descent
   parser for this language. The getExpr() method will use
   iteration, and it will call the getTerm() method. The getTerm()
   method will call the getExpr() method. So these two methods will
   be mutually recursive (though neither method is itself recursive).
*/

public class ParserL3
{
   public static Tree parseL(String stringInL3)
   {
      Tokenizer tokens = new Tokenizer(stringInL3);  // Tokenizer is defined in Tokenizer.java

      return getExpr(tokens);
   }//parseL()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = getTerm(tokens);
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         String tk = tokens.nextToken();         // consume the '+'
         Tree subExpr = getTerm(tokens);         // consume a sub expression
         result = new Tree(tk, result, subExpr); // old result on the left
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = null;
      String tk = tokens.peekToken();
      if( tk.equals("(") )             // conditionally consume more tokens
      {
         tokens.match("(");        // consume a '('
         result = getExpr(tokens); // consume a sub expression
         tokens.match(")");        // consume a ')'
      }
      else
      {
         result = new Tree(tokens.nextToken()); // consume the 'a'
      }
      return result;
   }//getTerm()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a", "( a )", "a + a",
                              "( a ) + a", "a + ( a )",
                              "( a + a )", "a + b + c",
                              "( a + b ) + c",
                              "a + ( b + c )",
                              "( ( a + b ) + c ) + d",
                              "a + ( b + ( c + d ) )",
                              "( ( a ) + ( ( b ) + ( ( c + d ) ) ) )" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parseL(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("ParserL3_%02d_AST", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//ParserL3