/*
   This example produces a full parse tree
   instead of an abstract syntax tree,
   as in the example ParseL3.java.

    L3 = {a, (a), a+a, (a)+a, a+(a), (a+a), a+a+a, (a+a)+a, a+(a+a), ((a+a)+a)+a, ...}


    EBNF:  expr -> term ( '+' term )*
           term -> a | '(' expr ')'
*/

public class ParserL3ToParseTree
{
   public static Tree parseL(String stringInL3)
   {
      Tokenizer tokens = new Tokenizer(stringInL3);  // Tokenizer is defined in Tokenizer.java

      return getExpr(tokens);
   }//parseL()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );;
      result.addSubTree( getTerm(tokens) );
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the '+'
         result.addSubTree( getTerm(tokens) );              // consume a sub expression
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = new Tree( "Term" );
      String tk = tokens.peekToken();
      if( tk.equals("(") )             // conditionally consume more tokens
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume a '('
         result.addSubTree( getExpr(tokens) );              // consume a sub expression
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume a ')'
      }
      else
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the 'a'
      }
      return result;
   }//getTerm()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a", "( a )", "a + a",
                              "( a ) + a", "a + ( a )",
                              "( a + a )", "a + a + a",
                              "( a + b ) + c",
                              "a + ( b + c )",
                              "( ( a + b ) + c ) + d",
                              "a + ( b + ( c + d ) )",
                              "( ( a ) + ( ( b ) + ( ( c + d ) ) ) )" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parseL(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("ParserL3_%02d_ParseTree", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//ParserL3ToParsTree