/*
   Here is a grammar for a language with two operators

   L4 = { a, a+a, a*a, a+a+a, a+a*a, a*a+a, a*a*a, ... }

   which gives '*' higher precedence than '+',
   makes '+' left associative,
   and makes '*' right associative.

    BNF:   expr -> expr '+' term | term   // expr is left-recursive
           term -> a '*' term | a         // term is right-recursive


    EBNF:  expr -> term ( '+' term )*     // expr is iterative
           term -> a [ '*' term ]         // term is still right-recursive
*/

public class ParserL4
{
   public static Tree parseL(String stringInL4)
   {
      Tokenizer tokens = new Tokenizer(stringInL4);

      return getExpr(tokens);
   }//parseL()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = getTerm(tokens);
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         String tk = tokens.nextToken();         // consume the '+'
         Tree subExpr = getTerm(tokens);         // consume a sub expression
         result = new Tree(tk, result, subExpr); // old result on the left
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = new Tree(tokens.nextToken());        // consume a 'a'
      if( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         String tk = tokens.nextToken();                 // consume the '*'
         result = new Tree(tk, result, getTerm(tokens)); // consume a sub expression
      }
      return result;
   }//getTerm()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a + b + c", "a + b * c",
                              "a * b + c", "a * b * c",
                              "a + b + c + e + f + g + h",  // sum of 7 terms
                              "a + b + c * e * f + g + h",  // sum of 5 terms
                              "a + b + c + e * f * g * h",  // sum of 4 terms
                              "a * b * c * e + f + g + h",  // sum of 4 terms
                              "a * b * c + e + f * g * h",  // sum of 3 terms
                              "a * b * c * e * f * g * h" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parseL(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("ParserL4_%02d_AST", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//ParserL4