/*
   This example produces a full parse tree
   instead of an abstract syntax tree,
   as in the example ParseL5.java.

   L5 = { a, a+a, a*a, a+(a+a), (a+a)*a, a*a+a, (a*a)*a, ... }

   '*' has higher precedence than '+',
   '+' is left associative,
   '*' is right associative.

    BNF:   expr -> expr '+' term | term       // expr is left-recursive
           term -> factor '*' term | factor   // term is right-recursive
         factor -> a | '(' expr ')'


    EBNF:  expr -> term ( '+' term )*        // expr is iterative
           term -> factor [ '*' term ]       // term is still right-recursive
         factor -> a | '(' expr ')'

   Notice that getExpr() is the same in this language as in L4,
               getTerm() is almost the same as getTerm() from L4,
               getFactor() is the same as getTerm() from L3.
*/

public class ParserL5ToParseTree
{
   public static Tree parseL(String stringInL4)
   {
      Tokenizer tokens = new Tokenizer(stringInL4);

      return getExpr(tokens);
   }//parseL()


   public static Tree getExpr(Tokenizer tokens)
   {
      Tree result = new Tree( "Expr" );;
      result.addSubTree( getTerm(tokens) );
      while( tokens.hasToken() && tokens.peekToken().equals("+") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the '+'
         result.addSubTree( getTerm(tokens) );              // consume a sub expression
      }
      return result;
   }//getExpr()


   public static Tree getTerm(Tokenizer tokens)
   {
      Tree result = new Tree( "Term" );;
      result.addSubTree( getFactor(tokens) );
      if( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the '*'
         result.addSubTree( getTerm(tokens) );              // consume a sub expression
      }
      return result;
   }//getTerm()


   public static Tree getFactor(Tokenizer tokens)
   {
      Tree result = new Tree( "Factor" );
      String tk = tokens.peekToken();
      if( tk.equals("(") )             // conditionally consume more tokens
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume a '('
         result.addSubTree( getExpr(tokens) );              // consume a sub expression
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume a ')'
      }
      else
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the 'a'
      }
      return result;
   }//getFactor()


   public static void main(String[] args)
   {
      String[] stringsInL = { "a + b + c", "a + ( b + c )",
                              "a * b + c", "a * ( b + c )",
                              "a * b * c", "( a * b ) * c",
                              "a + b + ( c + e + f ) + g + h",     // sum of 5 terms
                              "a + ( b + c ) * e * ( f + g + h )", // sum of 2 terms
                              "( a + b + c + e ) * f * g * h",     // product of 4 factors
                              "a * b * ( c * e + f + g + h ) ",    // product of 3 factors
                              "a * b * c + e + f * g * h",         // sum of 3 terms
                              "a * b * c * e * f * g * h",         // product of 7 factors
                              "a + b + c * d",     // put in lots of redundant parentheses
                              "( a + b ) + c * d",
                              "( a + b ) + ( c * d )",
                              "( a + b ) + ( ( c * d ) )",
                              "( ( a + b ) + ( ( c * d ) ) )",
                              "( ( ( a ) + ( b ) ) + ( ( ( c ) * ( d ) ) ) )" };


      for (int i = 0; i < stringsInL.length; i++)
      {
         Tree parseTree = parseL(stringsInL[i]);

         try  // Create dot and png files from the parse tree.
         {
            // Create the (empty) dot file.
            String baseName = String.format("ParserL5_%02d_ParseTree", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println("//  " + parseTree + "\n");
            out.println( Tree2dot.tree2dot(parseTree) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()

}//ParserL5ToParseTree